@@ -7,7 +7,10 @@
 
 #include "SensorTreeModel.h"
 
+#include <optional>
+
 #include <KLocalizedString>
+#include <QCollator>
 #include <QDebug>
 #include <QMetaEnum>
 #include <QMimeData>
@@ -23,38 +26,61 @@
 
 using namespace KSysGuard;
 
+struct Compare {
+    bool operator()(const QString &first, const QString &second) const
+    {
+        // Place "All" object at the top.
+        if (first == QLatin1String("all") && first != second) {
+            return true;
+        }
+
+        if (second == QLatin1String("all")) {
+            return false;
+        }
+
+        if (!collator) {
+            collator = QCollator();
+            collator->setNumericMode(true);
+            collator->setCaseSensitivity(Qt::CaseInsensitive);
+        }
+
+        return collator->compare(first, second) == -1;
+    }
+
+    // This uses thread-local storage because QCollator may not be thread safe.
+    // We store it in an optional to make sure we can initialize it above.
+    thread_local static std::optional<QCollator> collator;
+};
+
+thread_local std::optional<QCollator> Compare::collator = std::nullopt;
+
 struct Q_DECL_HIDDEN SensorTreeItem {
     SensorTreeItem *parent = nullptr;
     QString segment;
-    QMap<QString, SensorTreeItem *> children;
+    std::map<QString, std::unique_ptr<SensorTreeItem>, Compare> children;
 
-    inline int indexOf(const QString &segment)
+    inline int indexOf(const QString &segment) const
     {
-        int index = 0;
-        for (auto child : std::as_const(children)) {
-            if (child->segment == segment) {
-                return index;
-            }
-            index++;
+        auto itr = std::find_if(children.cbegin(), children.cend(), [segment](const auto &item) {
+            return item.second->segment == segment;
+        });
+
+        if (itr != children.cend()) {
+            return std::distance(children.cbegin(), itr);
         }
 
         return -1;
     }
 
-    inline SensorTreeItem *itemAt(int index)
+    inline SensorTreeItem *itemAt(std::size_t index) const
     {
-        int currentIndex = 0;
-        for (auto child : std::as_const(children)) {
-            if (currentIndex++ == index) {
-                return child;
-            }
+        if (index >= children.size()) {
+            return nullptr;
         }
-        return nullptr;
-    }
 
-    ~SensorTreeItem()
-    {
-        qDeleteAll(children);
+        auto itr = children.cbegin();
+        std::advance(itr, index);
+        return itr->second.get();
     }
 };
 
@@ -121,28 +147,25 @@ void SensorTreeModel::Private::addSensor(const QString &sensorId, const SensorIn
             addSensor(sensorIdExpr, newInfo);
         }
     }
+
     SensorTreeItem *item = rootItem;
-
     for (auto segment : segments) {
-        auto child = item->children.value(segment, nullptr);
-
-        if (child) {
-            item = child;
-
+        if (auto itr = item->children.find(segment); itr != item->children.end() && itr->second) {
+            item = itr->second.get();
         } else {
-            SensorTreeItem *newItem = new SensorTreeItem();
+            auto newItem = std::make_unique<SensorTreeItem>();
             newItem->parent = item;
             newItem->segment = segment;
 
             const QModelIndex &parentIndex = (item == rootItem) ? QModelIndex() : q->createIndex(item->parent->indexOf(item->segment), 0, item);
 
-            auto index = std::distance(item->children.begin(), item->children.upperBound(segment));
+            auto index = std::distance(item->children.begin(), item->children.upper_bound(segment));
 
             q->beginInsertRows(parentIndex, index, index);
-            item->children.insert(segment, newItem);
+            item->children[segment] = std::move(newItem);
             q->endInsertRows();
 
-            item = newItem;
+            item = item->children[segment].get();
         }
     }
 
@@ -176,7 +199,10 @@ void SensorTreeModel::Private::removeSensor(const QString &sensorId)
 
         const QModelIndex &parentIndex = (parent == rootItem) ? QModelIndex() : q->createIndex(parent->parent->indexOf(parent->segment), 0, parent);
         q->beginRemoveRows(parentIndex, index, index);
-        delete item->parent->children.take(item->segment);
+
+        auto itr = item->parent->children.find(item->segment);
+        item->parent->children.erase(itr);
+
         q->endRemoveRows();
 
         sensorInfos.remove(item);
@@ -184,7 +210,7 @@ void SensorTreeModel::Private::removeSensor(const QString &sensorId)
 
     remove(item, parent);
 
-    while (!parent->children.count()) {
+    while (!parent->children.size()) {
         item = parent;
         parent = parent->parent;
 
@@ -217,8 +243,9 @@ SensorTreeItem *KSysGuard::SensorTreeModel::Private::find(const QString &sensorI
     auto item = rootItem;
     const auto segments = sensorId.split(QLatin1Char('/'));
     for (const QString &segment : segments) {
-        item = item->children.value(segment, nullptr);
-        if (!item) {
+        if (auto itr = item->children.find(segment); itr != item->children.end() && itr->second) {
+            item = itr->second.get();
+        } else {
             return nullptr;
         }
     }
@@ -347,10 +374,10 @@ int SensorTreeModel::rowCount(const QModelIndex &parent) const
         }
 
         const SensorTreeItem *item = static_cast<SensorTreeItem *>(parent.internalPointer());
-        return item->children.count();
+        return item->children.size();
     }
 
-    return d->rootItem->children.count();
+    return d->rootItem->children.size();
 }
 
 int SensorTreeModel::columnCount(const QModelIndex &parent) const
@@ -372,7 +399,7 @@ QModelIndex SensorTreeModel::index(int row, int column, const QModelIndex &paren
         parentItem = static_cast<SensorTreeItem *>(parent.internalPointer());
     }
 
-    if (row < 0 || row >= parentItem->children.count()) {
+    if (row < 0 || row >= int(parentItem->children.size())) {
         return QModelIndex();
     }
 
